import { BaseError as CustomError } from 'make-error';
import { RetryOptions } from './retry';
import { IgnoreFunc } from './ignore';
/**
 * Base error class.
 */
export declare class BaseError<A> extends CustomError {
    /**
     * Initializes a new instance of the error.
     *
     * @param message Error message.
     * @param attrs Attributes to merge onto the instance.
     */
    constructor(message?: string, attrs?: Partial<ErrorAttributes<A>>);
    /**
     * Preconfigures an `assert` function.
     *
     * @param message Error message.
     * @param attrs Attributes to merge onto the instance.
     *
     * @example
     *   promise.then(MyError.makeAssert('Stuff was not found'))
     */
    static makeAssert<T, A>(message: string | (() => string), attrs?: Partial<ErrorAttributes<A>>): Asserter<T>;
    /**
     * If `data` is falsy, throws the error with the specified message.
     *
     * @param data The data to check.
     * @param message The message to construct the error with.
     * @param attrs Attributes to merge onto the instance.
     */
    static assert<T, A>(data: T, message?: string | (() => string), attrs?: Partial<ErrorAttributes<A>>): asserts data;
    /**
     * Retries the function until it does not throw the error type.
     * @param fn
     * @param opts
     */
    static retry<T>(fn: (attempt: number) => Promise<T>, opts?: Partial<RetryOptions>): Promise<T>;
    /**
     * Makes an ignore function for this error class.
     */
    static ignore(): IgnoreFunc<undefined>;
    /**
     * Makes an ignore function for this error class that will return the specified value if caught.
     *
     * @param valueToReturnOnCatch
     */
    static ignore<T>(valueToReturnOnCatch: T): IgnoreFunc<T>;
    /**
     * Default toJSON implementation.
     */
    toJSON(): Object;
}
/**
 * Asserter function.
 */
export declare type Asserter<T> = (value: T) => NonFalsy<T>;
/**
 * Non-falsy value.
 */
export declare type NonFalsy<T> = Exclude<T, false | 0 | undefined | null | ''>;
/**
 * Error attributes.
 */
export declare type ErrorAttributes<A> = A & {
    [key: string]: any;
};
/**
 * Base error constructor.
 */
export interface BaseErrorConstructor<A> {
    /**
     * Constructs the error.
     */
    new (message?: string, attrs?: Partial<ErrorAttributes<A>>): BaseError<A> & ErrorAttributes<A>;
    /**
     * Makes an asserter function.
     */
    makeAssert<T>(message: string | (() => string), attrs?: Partial<ErrorAttributes<A>>): Asserter<T>;
    /**
     * Asserts the truthiness of the given value, throws the error otherwise.
     */
    assert<T>(data: T, message?: string | (() => string), attrs?: Partial<ErrorAttributes<A>>): asserts data;
    /**
     * Retries the function until it does not throw the error type.
     */
    retry<T>(fn: (attempt: number) => Promise<T>, opts?: Partial<RetryOptions>): Promise<T>;
    /**
     * Makes an ignore function for this error class.
     */
    ignore(): IgnoreFunc<undefined>;
    /**
     * Makes an ignore function for this error class that will return the specified value if caught.
     *
     * @param valueToReturnOnCatch
     */
    ignore<T>(valueToReturnOnCatch: T): IgnoreFunc<T>;
    /**
     * Makes an ignore function for this error class that will return the specified value if caught.
     *
     * @param valueToReturnOnCatch
     */
    ignore(valueToReturnOnCatch?: any): IgnoreFunc<any>;
}
/**
 * Makes a new error class preconfigured with the specified settings,
 * as well as some extra methods for assertions.
 *
 * @param defaultMessage The default error message.
 * @param defaultAttrs The default attributes for creating new instances.
 */
export declare function MakeErrorClass<A = {}>(defaultMessage?: string, defaultAttrs?: A): BaseErrorConstructor<A>;
